var listOfAllMessageItems = [];

function lpad(s) {
    return ("0" + s).slice(-2);
}

/**
 * Adds an entry to the message list
 */
function addMessageToList(msgaKey, msg, parent, direction) {
    if (parent.children.length == 1) {
        var del = parent.children[0];
        if (del) {
            var shouldDelete = del.getAttribute("x-del");
            if (shouldDelete === "yes") {
                parent.removeChild(del);
            }
        }
    }
    var el = document.createElement("div");
    el.setAttribute("class", "small-12 cell msgArchiveItem animated bounceInRight");
    el.classList.add(msgaKey);
    el.setAttribute("x-key", msg["token"]);


    var elItemContainer = document.createElement("div");
    elItemContainer.insertAdjacentHTML('beforeend', '<div class="icon"><i class="fa fa-ico-warning fa-2x"></i></div>');

    // Set notification status
    if(msgaKey == 'MA@statusmeldung'){
        elItemContainer.insertAdjacentHTML('beforeend', '<h2>Statusmeldung</h2>');
        elItemContainer.setAttribute("class", "item notification success");
    }
    if(msgaKey == 'MA@stoermeldung'){
        elItemContainer.insertAdjacentHTML('beforeend', '<h2>Störmeldung</h2>');
        elItemContainer.setAttribute("class", "item notification alert");
    }
    if(msgaKey == 'MA@warnmeldung'){
        elItemContainer.insertAdjacentHTML('beforeend', '<h2>Warnmeldung</h2>');
        elItemContainer.setAttribute("class", "item notification warning");
    }

    el.appendChild(elItemContainer);

    var elTS = document.createElement("small");
    elTS.classList.add("msgTS");
    var ts = new Date(msg["ts"] * 1000);
    elTS.innerHTML = lpad(ts.getDate()) + "." + lpad(ts.getMonth() + 1) + "." + lpad(ts.getFullYear()) + ", " + lpad(ts.getHours()) + ":" + lpad(ts.getMinutes()) + ":" + lpad(ts.getSeconds());
    elItemContainer.appendChild(elTS);

    var elText = document.createElement("p");
    elText.classList.add("msgText");
    elText.innerHTML = msg["text"];
    elItemContainer.appendChild(elText);
    if (direction == 0)
        parent.appendChild(el);
    else
        parent.insertBefore(el, parent.firstChild);
    var max = parseInt(parent.getAttribute("x-max") || "50");
    while (parent.children.length > max)
        parent.removeChild(parent.lastChild);
}

/**
 * Function for sorting messages according to their timestamp.
 */
function sortMessages(x, y) {
    if (x[1]["ts"] < y[1]["ts"])
        return 1;
    else if (x[1]["ts"] > y[1]["ts"])
        return -1;
    return 0;
}

/**
 * Is called as soon as a new message arrives. Displays the messages sorted by time.
 */
function onMessageArchive(err, key, data, ctx) {
    var parent = document.getElementById("viewMsgA");
    while (parent.firstChild != null)
        parent.removeChild(parent.firstChild);
    if (!err && data) {
        for (var i = 0; i < data.length; i++) {
            listOfAllMessageItems.push([key, data[i]]);
        }
        listOfAllMessageItems = listOfAllMessageItems.sort(sortMessages);
        for (var i = 0; i < listOfAllMessageItems.length; i++) {
            addMessageToList(listOfAllMessageItems[i][0], listOfAllMessageItems[i][1], parent, 0);
        }
    }
}

/**
 * Retrieves all message archives. As soon as a new message arrives in one of the archives,
 * it is displayed.
 * @param {type} keyList List of message archive keys to be queried
 */
function requestMessageArchives(keyList) {
    keyList.forEach(function (key) {
        var ma = conn.getMessageArchive(key, function (err, data) {
            this.getData(1, function (err, items) {
                addMessageToList(key, items[0], document.getElementById("viewMsgA"), 1, true);
            });

        });
        ma.getData(10, function (err, items) {
            onMessageArchive(err, key, items);
        });

    });
}
